C*  polder.h
C*  author: CESBIO / CISI / CNES
C* 
C* LIBPOL_VERSION  "@(#)polder.h	1.3 98/01/16"  
#ifndef _POLDER_H_
#define _POLDER_H_


C*  Definition of error codes returned by functions   
C*  -----------------------------------------------   
#define PLD_OK			0
#define PLD_UNDEF_PID		-1
C* Product identifiant undefined  
#define PLD_MAX_OPENED		-2   
C* too many products open  
#define PLD_FILE_ACCESS		-3   
C* error on opening leader file  
#define PLD_ALT_FILENAME	-4   
C* not generic filename  
#define PLD_FILESIZE		-5   
C* error on accessing data  
#define PLD_PRODUCT_EMPTY	-6   
C* product empty  
#define PLD_PIXEL_UNDEFINED	-7   
C* pixel not observed  
#define PLD_NOMEM		-8   
C* memory allocation error  
#define PLD_INVAL		-9   
C* invalid parameter  
#define PLD_OUTSIDE		-10  
C* line outside the product  
#define PLD_LIG_INVAL		-11  
C* invalid lines ( first line > Last line) 
#define PLD_PRODUCT_UNAVAILABLE	-12
C* product and libpol are not compatible 

C*  Definition of interleaving mode   
C*  -------------------------------   
#define BIP	0	
C* parameter values are organized by pixel: 
C* line 1 (pixel1: par.1, par.2,... / pixel2: par.1, ...)
C* line 2 (pixel1: par.1, par.2,... / pixel2: par.1, ...)
C*... 
 
#define BIL	1       
C* parameter values are organized by line:
C* line1 (pixel1: par.1, pixel2: par.1, ..., pixeln: par.1)
C* line1 (pixel1: par.2, pixel2: par.2, ..., pixeln: par.2)
C* ...
C* line2 (pixel1: par.1, pixel2: par.1, ..., pixeln: par.1)
C* line2 (pixel1: par.2, pixel2: par.2, ..., pixeln: par.2)
C* ...
   
#define BSQ	2
C* parameter values are organized by parameter:
C* line1 (pixel1: par.1, pixel2: par.1, ..., pixeln: par.1)
C* line2 (pixel1: par.1, pixel2: par.1, ..., pixeln: par.1)
C* ...
C* line1 (pixel1: par.2, pixel2: par.2, ..., pixeln: par.2)
C* line2 (pixel1: par.2, pixel2: par.2, ..., pixeln: par.2)
C* ...   

#define NaN		-999.

C*  Definition of extraction mode for directional parameters   
C*  --------------------------------------------------------   
#define RAD2REF		268435456
C* extract reflectance in place of radiance					
C** works with all extraction mode but LIST_+
C** usefull only with Level 1 product parameters  
#define VIEW_01		(1*65536)	
C* the value of the first direction . inactive  
#define VIEW_02		(2*65536)	
C* the value of the second direction. inactive 
#define VIEW_03		(3*65536)
#define VIEW_04		(4*65536)
#define VIEW_05		(5*65536)
#define VIEW_06		(6*65536)
#define VIEW_07		(7*65536)
#define VIEW_08		(8*65536)	
C* ...  
#define VIEW_09		(9*65536)	
C* the value may be undefined . inactive 
#define VIEW_10		(10*65536)
#define VIEW_11		(11*65536)
#define VIEW_12		(12*65536)
#define VIEW_13		(13*65536)
#define VIEW_14		(14*65536)	
C* value of the fourteenth direction  
#define ALL		(1*1048576)	
C* all values  
#define FIRST		(2*1048576)	
C* first valid parameter value  
#define LAST		(3*1048576)	
C* last valid parameter value  
#define MEDIAN		(4*1048576)	
C* value for median viewing configuration. inactive 
#define MEAN		(5*1048576)	
C* mean of valid values  
#define NADIR		(6*1048576)	
C* value for nearest nadir direction  
#define LIST_01		(1*16777216)	
C* first parameter value (valid or not)  
#define LIST_02		(2*16777216)	
C* two parameters that follow itself  
#define LIST_03		(3*16777216)
#define LIST_04		(4*16777216)
#define LIST_05		(5*16777216)
C* Note: LIST_01 is default mode if omited in the argument list for a directional parameter  

C*  Definition of parameter types classed by product   
C*  ------------------------------------------------   
C*  Note: the parameters with '!' in the description does not be extracted with
C*  	ExtractPixel or ExtractBlock functions.
 
#define UNDEF		-1	
C* Parameter undefined  

C*  LEVEL 1   
C*----------- 
#define B11		4
#define B11_NB_PAR_DIR	23		
C* number of directional parameters  
#define	SB		1073741824	
C* signed word encoding  

#define	B1_DQX		0	
C* ! pixel confidence data  
#define	B1_CI		1	
C* rough cloud indicator  
#define	B1_AZS		2	
C* solar azimuth angle  
#define	B1_NVD		3	
C* number of available viewing directions  
#define	B1_SA		4	
C* ! sequence arrangement  
	
C* directional parameters  
#define	B1_SN		5	
C* sequence number within the orbit 
#define	B1_LN		(6+SB)	
C* CCD matrix detector line number  
#define	B1_CN		(7+SB)	
C* CCD matrix detector coulmn number  
#define	B1_THS		8	
C* solar zenith solar angle  
#define	B1_THV		9	
C* view zenith angle  
#define	B1_RAZ		10	
C* relative azimuthal angle  
#define	B1_DC_THV	(11+SB)
#define	B1_DS_THV	(12+SB)
#define	B1_I443		(13+SB)	
C* normalized radiance for channel 443 nm unpolarized  
#define	B1_I443P	(14+SB)	
C* normalized radiance for channel 443 nm polarized  
#define	B1_I490		(15+SB)	
C* normalized radiance for channel 490 nm  
#define	B1_I565		(16+SB)	
C* normalized radiance for channel 565 nm  
#define	B1_I670P	(17+SB)	
C* normalized radiance for channel 670 nm polarized  
#define	B1_I763		(18+SB)	
C* normalized radiance for channel 763 nm  
#define	B1_I765		(19+SB)	
C* normalized radiance for channel 765 nm  
#define	B1_I865P	(20+SB)	
C* normalized radiance for channel 865 nm polarized  
#define	B1_I910		(21+SB)	
C* normalized radiance for channel 910 nm  
#define	B1_Q443P	(22+SB)	
C* second component of Stockes vector for channel 443 nm polarized  
#define	B1_Q670P	(23+SB)	
C* second component of Stockes vector for channel 670 nm polarized  
#define	B1_Q865P	(24+SB)	
C* second component of Stockes vector for channel 865 nm polarized  
#define	B1_U443P	(25+SB)	
C* third component of Stockes vector for channel 443 nm polarized  
#define	B1_U670P	(26+SB)	
C* third component of Stockes vector for channel 670 nm polarized  
#define	B1_U865P	(27+SB)	
C* third component of Stockes vector for channel 865 nm polarized  

C*  LEVEL 2   
C*  Land Surfaces & Atmosphere: surface parameters   
C*-------------------------------------------------- 
#define L2A		24
#define L2A_NB_PAR_DIR	9	
C* number of directional parameters  
#define	L2_DQX		0	
C* ! pixel confident data  
#define	L2_THS		1	
C* solar zenith angle  
#define	L2_AZS		2	
C* solar azimuth angle  
#define	L2_NVD		3	
C* number of viewing directions  

C* directional parameters  
#define	L2_SN		4	
C* sequence number within the orbit  
#define	L2_THV		5	
C* view zenith angle  
#define	L2_RAZ		6	
C* relative azimuth angle  
#define	L2_R443		7	
C* surface reflectance for channel 443 nm  
#define	L2_R565		8	
C* surface reflectance for channel 565 nm  
#define	L2_R670		9	
C* surface reflectance for channel 670 nm  
#define	L2_R765		10	
C* surface reflectance for channel 765 nm  
#define	L2_R865		11	
C* surface reflectance for channel 865 nm  
#define	L2_RP865	12	
C* signed polarised reflectance for channel 865 nm  

C*  LEVEL 2   
C*  Land Surfaces & Atmosphere: aerosols and water vapor   
C*-------------------------------------------------------- 
#define L2C		25
#define	L2_DQX		0	
C* ! pixel confident data  
#define	L2_TAU865	1	
C* aerosol optical thickness for 865 nm  
#define	L2_INDEX	2	
C* particle refractive index  
#define	L2_ANGST	3	
C* aerosol Angstrom coefficient  
#define	L2_IA		4	
C* aerosol index  
#define	L2_TAU865F	5	
C* optical thickness, from fixed mode, for 865 nm  
#define	L2_IQMES	6	
C* measurement qualty index  
#define	L2_IQINV	7	
C* polarized radiance inversion quality index  	

C*  LEVEL 3   
C*  Land Surfaces & Atmosphere: surface directional signatures   
C*-------------------------------------------------------------- 
#define L3A		28
#define	L3_DQX		0	
C* ! pixel confident data  
#define	L3_TH0		1	
C* solar zenith angle  
#define	L3_K443_1	2	
C* first parameter BRDF model for channel 443 nm  
#define	L3_K443_2	3	
C* second parameter BRDF model for channel 443 nm  
#define	L3_K443_3	4	
C* ...  
#define	L3_ER_K443_1	5	
C* Accuracy of the 1st parameter BRDF model for channel 443 nm  
#define	L3_ER_K443_2	6	
C* Accuracy of the 2nd parameter BRDF model for channel 443 nm  
#define	L3_ER_K443_3	7	
C* ...  
#define	L3_K565_1	8	
C* first parameter BRDF model for channel 565 nm  
#define	L3_K565_2	9	
C* second parameter BRDF model for channel 565 nm  
#define	L3_K565_3	10	
C* ...  
#define	L3_ER_K565_1	11	
C* Accuracy of the 1st parameter BRDF model for channel 565 nm  
#define	L3_ER_K565_2	12	
C* Accuracy of the 2nd parameter BRDF model for channel 565 nm  
#define	L3_ER_K565_3	13	
C* ...  
#define	L3_K670_1	14	
C* first parameter BRDF model for channel 670 nm  
#define	L3_K670_2	15	
C* second parameter BRDF model for channel 670 nm  
#define	L3_K670_3	16	
C* ...  
#define	L3_ER_K670_1	17	
C* Accuracy of the 1st parameter BRDF model for channel 670 nm  
#define	L3_ER_K670_2	18	
C* Accuracy of the 2nd parameter BRDF model for channel 670 nm  
#define	L3_ER_K670_3	19	
C* ...  
#define	L3_K765_1	20	
C* first parameter BRDF model for channel 765 nm  
#define	L3_K765_2	21	
C* second parameter BRDF model for channel 765 nm  
#define	L3_K765_3	22	
C* ...  
#define	L3_ER_K765_1	23	
C* Accuracy of the 1st parameter BRDF model for channel 765 nm  
#define	L3_ER_K765_2	24	
C* Accuracy of the 2nd parameter BRDF model for channel 765 nm  
#define	L3_ER_K765_3	25	
C* ...  
#define	L3_K865_1	26	
C* first parameter BRDF model for channel 865 nm  
#define	L3_K865_2	27	
C* second parameter BRDF model for channel 865 nm  
#define	L3_K865_3	28	
C* ...  
#define	L3_ER_K865_1	29	
C* Accuracy of the 1st parameter BRDF model for channel 865 nm  
#define	L3_ER_K865_2	30	
C* Accuracy of the 2nd parameter BRDF model for channel 865 nm  
#define	L3_ER_K865_3	31	
C* ...  

C*  LEVEL 3   
C*  Land Surfaces & Atmosphere: albedo and NDVI   
C*----------------------------------------------- 
#define L3B		29
#define	L3_BDQX		0	
C* ! pixel confident data  
#define	L3_BTH0		1	
C* solar zenith angle  
#define	L3_RH443	2	
C* directional hemispheric reflectance for channel 443 nm  
#define	L3_ER_RH443	3	
C* Accuracy of th e directional hemispheric reflectance for channel 443 nm 
#define	L3_RH565	4	
C* directional hemispheric reflectance for channel 565 nm  
#define	L3_ER_RH565	5	
C* Accuracy of th e directional hemispheric reflectance for channel 565 nm 
#define	L3_RH670	6	
C* directional hemispheric reflectance for channel 670 nm  
#define	L3_ER_RH670	7	
C* Accuracy of th e directional hemispheric reflectance for channel 670 nm 
#define	L3_RH765	8	
C* directional hemispheric reflectance for channel 765 nm  
#define	L3_ER_RH765	9	
C* Accuracy of th e directional hemispheric reflectance for channel 765 nm 
#define	L3_RH865	10	
C* directional hemispheric reflectance for channel 865 nm  
#define	L3_ER_RH865	11	
C* Accuracy of th e directional hemispheric reflectance for channel 865 nm 
#define	L3_NDVI		12	
C* Normalised Difference Vegetation Index  
#define	L3_ER_NDVI	13	
C* Accuracy of Normalised Difference Vegetation Index  
#define	L3_LAI		14	
C* Leaf Area Index  
#define	L3_ER_LAI	15	
C* Accuracy of Leaf Area Index  
#define	L3_VEG		16	
C* Vegetation cover  
#define	L3_ER_VEG	17	
C* Accuracy of Vegetation cover  

C*  LEVEL 3   
C*  Land Surfaces & Atmosphere: atmospheric parameters  
C*----------------------------------------------------- 
#define L3C		30
#define L3_CDQX         0       
C* ! pixel confident data  
#define L3_NB_1		1	
C* number of observations used within decade 1  
#define L3_TAU_1	2	
C* mean general aerosol optical thickness within decade 1  
#define L3_ANGST_1	3	
C* mean of angstrom coefficient within decade 1  
#define L3_IA_1		4	
C* mean aerosol index within decade 1 
#define L3_TAUF_1	5	
C* mean aerosol optical thickness from fixed mode within decade 1  
#define L3_NB_2		6	
C* number of observations used within decade 2  
#define L3_TAU_2	7	
C* mean general aerosol optical thickness within decade 2  
#define L3_ANGST_2	8	
C* mean of angstrom coefficient within decade 2  
#define L3_IA_2		9	
C* mean aerosol index within decade 2 
#define L3_TAUF_2	10	
C* mean aerosol optical thickness from fixed mode within decade 2  
#define L3_NB_3		11	
C* number of observations used within decade 3  
#define L3_TAU_3	12	
C* mean general aerosol optical thickness within decade 3  
#define L3_ANGST_3	13	
C* mean of angstrom coefficient within decade 3  
#define L3_IA_3		14	
C* mean aerosol index within decade 3 
#define L3_TAUF_3	15	
C* mean aerosol optical thickness from fixed mode within decade 3  
#define L3_NB_M		16	
C* number of observations used within the month  
#define L3_TAU_M	17	
C* mean general aerosol optical thickness within the month  
#define L3_ANGST_M	18	
C* mean of angstrom coefficient within the month   
#define L3_IA_M		19
C* mean aerosol index within the month 
#define L3_TAUF_M	20		
C* mean aerosol index within the month 
#define L3_TAUQ_0	21      
C* minimal  general aerosol optical thickness   
#define L3_TAUQ_1	22      
C* first quartile of  general aerosol optical thickness   
#define L3_TAUQ_2	23      
C* mean general aerosol optical thickness   
#define L3_TAUQ_3	24      
C* third quartile of general aerosol optical thickness    
#define L3_TAUQ_4	25	
C* maximal general aerosol optical   
#define L3_IAQ_0	26      
C* minimal  aerosol index   
#define L3_IAQ_1	27      
C* first quartile of  aerosol index   
#define L3_IAQ_2	28      
C* mean  aerosol index   
#define L3_IAQ_3	29      
C* third quartile of  aerosol index    
#define L3_IAQ_4	30	
C* maximal  aerosol index   
#define L3_TAUQF_0	31      
C* minimal  aerosol optical thickness  from fixed mode  
#define L3_TAUQF_1	32      
C* first quartile of  aerosol optical thickness from fixed mode   
#define L3_TAUQF_2	33      
C* mean aerosol optical thickness  from fixed mode 
#define L3_TAUQF_3	34      
C* third quartile of aerosol optical thickness  from fixed mode  
#define L3_TAUQF_4	35	
C* maximal aerosol optical from fixed mode  
#define L3_F_ANGST_1	36	
C* frequency of angstrom coefficient occurence in class 1  
#define L3_F_ANGST_2	37      
C* frequency of angstrom coefficient occurence in class 2  
#define L3_F_ANGST_3	38      
C* frequency of angstrom coefficient occurence in class 3  
#define L3_F_ANGST_4	39      
C* frequency of angstrom coefficient occurence in class 4  
#define L3_F_INDEX_1	40	
C* frequency of refractive index occurence in class 1  
#define L3_F_INDEX_2	41      
C* frequency of refractive index occurence in class 2  
#define L3_F_INDEX_3	42      
C* frequency of refractive index occurence in class 3  

C*  LEVEL 2   
C*  Ocean color & marine aerosols: directional parameters   
C*--------------------------------------------------------- 
#define O2A		40
#define O2A_NB_PAR_DIR	8	
C* number of directional parameters  
#define	O2_DQX		0	
C* ! pixel confident data  
#define	O2_THS		1	
C* solar zenith angle  
#define	O2_AZS		2	
C* solar azimuth angle  
#define O2_NVD		3	
C* number of viewing directions  
#define	O2_X_DISX	4	
C* SeaWIFS aerosol model  
#define	O2_AANGST	5	
C* Angstrom coefficient  
#define	O2_ATAU865	6	
C* aerosol optical thickness for 865 nm  
#define	O2_AOMEGA	7	
C* albedo for a simple diffusion  
#define	O2_FOAMREFL	8	
C* foam reflectance  
#define	O2_IQAERO	9	
C* aerosols confidence index  
#define	O2_TICTIL	10	
C* integration time flags  
#define	O2_UO3		11	
C* ozone integrated content  
#define	O2_WINDVEL	12	
C* wind surface velocity  
#define	O2_SURFPRES	13	
C* surface pressure  
	
C* directional parameters  
#define	O2_THV		14	
C* view zenith angle  
#define	O2_RAZ		15	
C* relative azimuth angle  
#define	O2_GAMMA	16	
C* diffusion angle  
#define	O2_RW443	17	
C* marine reflectance for channel 443 nm  
#define	O2_RW490	18	
C* marine reflectance for channel 490 nm  
#define	O2_RW565	19	
C* marine reflectance for channel 565 nm  
#define	O2_RW670	20	
C* marine reflectance for channel 670 nm  
#define	O2_RC865	21	
C* reflectance for channel 865 nm  


C*  LEVEL 2   
C*  Ocean color & marine aerosols: non directional parameters   
C*------------------------------------------------------------- 
#define O2B		41
#define	O2_BDQX		0	
C* ! pixel confident data  
#define	O2_BTHS		1	
C* solar zenith angle  
#define	O2_BAZS		2	
C* solar azimuth angle  
#define	O2_BX_DISX	3	
C* SeaWIFS aerosol model  
#define	O2_BANGST	4	
C* Angstrom coefficient  
#define	O2_TAU865	5	
C* aerosol optical thickness for 865 nm  
#define	O2_BOMEGA	6	
C* Albedo for a simple diffusion  
#define	O2_BFOAMREFL	7	
C* foam reflectance  
#define	O2_BIQAERO	8	
C* aerosols confidence index  
#define	O2_DR443	9	
C* diffuse marine reflectance for channel 443 nm  
#define	O2_DR490	10	
C* diffuse marine reflectance for channel 490 nm  
#define	O2_DR565	11	
C* diffuse marine reflectance for channel 565 nm  
#define	O2_DR670	12	
C* diffuse marine reflectance for channel 670 nm  
#define	O2_PIGMENT	13	
C* pigment concentration  
#define	O2_CHLORO	14	
C* chlorophyll concentration (sea wifs model)  
#define	O2_WTYPE	15	
C* water type  
#define	O2_IQPIG	16	
C* pigment confidence index  
#define O2_BBACK	17	
C* back diffusion marine coefficient, for 443 nm  
#define	O2_ABS443	18	
C* absporption marine coefficient for 443 nm  
#define	O2_ABS490	19	
C* absporption marine coefficient for 490 nm  
#define	O2_WSURF	20	
C* surface wind  

C*  LEVEL 2   
C*  Ocean color & marine aerosols: marine aerosol parameters   
C*------------------------------------------------------------ 
#define O2C		42

#define O2_DQS		0	
C* ! super-pixel confident data 
#define O2_IQINVERS	1	
C* inversion quality index 
#define	O2_CTHS		2	
C* solar zenith angle 
#define O2_TAU865T	3       
C* total aerosols optical thickness, for 865 nm 
#define O2_TAU670T	4       
C* total aerosols optical thickness, for 670 nm 
#define O2_CANGST	5       
C* Angstrom coefficient from mix between 670 and 865 nm
#define O2_OMEGA	6	
C* mixing simple diffusion albedo for 865nm 
#define O2_G		7	
C* mixing asymetry factor 
#define O2_INDEX	8	
C* aerosol index 
#define	O2_REFF		9	
C* mixed mode real radius 
#define	O2_REFF_SMALL	10	
C* small mode real radius 
#define	O2_REFF_BIG	11	
C* big mode real radius 
#define O2_TAU865_SMALL	12      
C* aerosols optical thickness, from small model, for 865 nm 
#define O2_TAU670_SMALL	13      
C* aerosols optical thickness, from small model, for 670 nm 
#define O2_ANGST_SMALL	14	
C* Angstrom coefficient from mix between 670 and 865 nm small mode 
#define O2_TAU865_GS	15      
C* Tau of spherical coarse mode at 865.For favorable viewing geom
#define O2_TAU865_GNS	16      
C* Tau of non-spherical coarse mode at 865.For favorable viewing geom
#define	O2_ID		17	
C* Noshericity ratio of the big mode 
#define	O2_INDEX_SMALL	18	
C* small mode aerasols refractive index 
#define	O2_INDEX_BIG	19	
C* big mode aerasols refractive index 




C*  LEVEL 3   
C*  Ocean color & marine aerosols: marine parameters synthesis  
C*------------------------------------------------------------- 
#define O3B		45
#define O3_DQX          0       
C*! pixel confident data  
		
C* decade 1  
#define O3_MRAU443_D1	1	
C* mean of diffuse marine reflectances, for 443 nm  
#define O3_MRAU490_D1	2	
C* mean of diffuse marine reflectances, for 490 nm  
#define O3_MRAU565_D1	3	
C* mean of diffuse marine reflectances, for 565 nm  
#define O3_MRAU670_D1	4	
C* mean of diffuse marine reflectances, for 670 nm  
#define O3_MPIGMENT_D1	5	
C* mean of pigment concentrations  
#define O3_MCHLORO_D1	6	
C* mean of seaWIFS chlorophyll 
#define O3_MTAU865_D1	7	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_MANGST_D1	8	
C* mean of angstrom coefficient  
#define O3_MANG_TAU_D1	9	
C* mean of dust index 
#define O3_MOMEGA_D1	10	
C* mean of albedo for a simple diffusion  
#define O3_MBACK_D1	11	
C* mean of back diffuse coefficient  
#define O3_MABS443_D1	12	
C* mean of marine absorption coefficient, for 443 nm  
#define O3_MABS490_D1	13	
C* mean of marine absorption coefficient, for 490 nm  
#define O3_NOBS_D1	14	
C* number of observations 
#define O3_NOBSWATER_D1	15	
C* number of observation for water type 1  
#define O3_IQPIG_D1	16	
C* pigment confidence index  
		
C* decade 2  
#define O3_MRAU443_D2	17	
C* mean of diffuse marine reflectances, for 443 nm  
#define O3_MRAU490_D2	18	
C* mean of diffuse marine reflectances, for 490 nm  
#define O3_MRAU565_D2	19	
C* mean of diffuse marine reflectances, for 565 nm  
#define O3_MRAU670_D2	20	
C* mean of diffuse marine reflectances, for 670 nm  
#define O3_MPIGMENT_D2	21	
C* mean of pigment concentrations  
#define O3_MCHLORO_D2	22	
C* mean of seaWIFS chlorophyll 
#define O3_MTAU865_D2	23	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_MANGST_D2	24	
C* mean of angstrom coefficient  
#define O3_MANG_TAU_D2	25	
C* mean of dust index 
#define O3_MOMEGA_D2	26	
C* mean of albedo for a simple diffusion  
#define O3_MBACK_D2	27	
C* mean of back diffuse coefficient  
#define O3_MABS443_D2	28	
C* mean of marine absorption coefficient, for 443 nm  
#define O3_MABS490_D2	29	
C* mean of marine absorption coefficient, for 490 nm  
#define O3_NOBS_D2	30	
C* number of observations 
#define O3_NOBSWATER_D2	31	
C* number of observation for water type 1  
#define O3_IQPIG_D2	32	
C* pigment confidence index  
		
C* decade 3  
#define O3_MRAU443_D3	33	
C* mean of diffuse marine reflectances, for 443 nm  
#define O3_MRAU490_D3	34	
C* mean of diffuse marine reflectances, for 490 nm  
#define O3_MRAU565_D3	35	
C* mean of diffuse marine reflectances, for 565 nm  
#define O3_MRAU670_D3	36	
C* mean of diffuse marine reflectances, for 670 nm  
#define O3_MPIGMENT_D3	37	
C* mean of pigment concentrations  
#define O3_MCHLORO_D3	38	
C* mean of seaWIFS chlorophyll 
#define O3_MTAU865_D3	39	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_MANGST_D3	40	
C* mean of angstrom coefficient  
#define O3_MANG_TAU_D3	41	
C* mean of dust index 
#define O3_MOMEGA_D3	42	
C* mean of albedo for a simple diffusion  
#define O3_MBACK_D3	43	
C* mean of back diffuse coefficient  
#define O3_MABS443_D3	44	
C* mean of marine absorption coefficient, for 443 nm  
#define O3_MABS490_D3	45	
C* mean of marine absorption coefficient, for 490 nm  
#define O3_NOBS_D3	46	
C* number of observations 
#define O3_NOBSWATER_D3	47	
C* number of observation for water type 1  
#define O3_IQPIG_D3	48	
C* pigment confidence index  
		
C* month  
#define O3_MRAU443	49	
C* mean of diffuse marine reflectances, for 443 nm  
#define O3_MRAU490	50	
C* mean of diffuse marine reflectances, for 490 nm  
#define O3_MRAU565	51	
C* mean of diffuse marine reflectances, for 565 nm  
#define O3_MRAU670	52	
C* mean of diffuse marine reflectances, for 670 nm  
#define O3_ERAU443	53	
C* standard deviation of diffuse marine reflectances, for 443 nm  
#define O3_ERAU490	54	
C* standard deviation of diffuse marine reflectances, for 490 nm  
#define O3_ERAU565	55	
C* standard deviation of diffuse marine reflectances, for 565 nm  
#define O3_ERAU670	56	
C* standard deviation of diffuse marine reflectances, for 670 nm  
#define O3_MPIGMENT	57	
C* mean of pigment concentrations  
#define O3_MCHLORO	58	
C* mean of seaWIFS chlorophyll 
#define O3_EPIGMENT	59	
C* standard deviation of pigment concentrations  
#define O3_ECHLORO	60	
C* standard deviation of  seaWIFS chlorophyll 
#define O3_MINPIGMENT   61	
C* minimum of pigment concentration  
#define O3_MAXPIGMENT	62	
C* maximum of pigment concentration  
#define O3_MINCHLORO	63	
C* minimum of seaWIFS chlorophyll 
#define O3_MAXCHLORO	64	
C* maximum of seaWIFS chlorophyll 
#define O3_MTAU865	65	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_ETAU865	66	
C* standard deviation of aerosol optical thickness, for 865 nm  
#define O3_MINTAU865	67	
C* minimum of aerosol optical thickness, for 865 nm  
#define O3_MAXTAU865	68	
C* maximum of aerosol optical thickness, for 865 nm  
#define O3_MANGST	69	
C* mean of angstrom coefficient for a month 
#define O3_EANGST	70	
C* standard deviation of angstrom coefficient  
#define O3_MANG_TAU	71	
C* mean of dust index for a month 
#define O3_EANG_TAU	72	
C* standard deviation of dust index  
#define O3_MOMEGA	73	
C* mean of a simple diffusion albedo for a month 
#define O3_EOMEGA	74	
C* standard deviation a simple diffusion albedo  
#define O3_MBACK	75	
C* mean of back diffuse coefficient for a month 
#define O3_MABS443	76	
C* mean of marine absorption coefficient for a month, for 443 nm  
#define O3_MABS490	77	
C* mean of marine absorption coefficient for a month, for 490 nm  
#define O3_EBACK	78	
C* standard deviation of back diffuse coefficient  
#define O3_EABS443	79	
C* standard deviation of marine absorption coefficient , for 443 nm  
#define O3_EABS490	80	
C* standard deviation of marine absorption coefficient , for 490 nm  
#define O3_NOBS		81	
C* number of observations  
#define O3_NOBSWATER	82	
C* number of observation for water type 1  
#define O3_IQPIG	83	
C* pigment confidence index  

C*  LEVEL 3   
C*  Ocean color & marine aerosols: marine aerosol parameters   
C*------------------------------------------------------------ 
#define O3C		46
#define O3_DQS		0	
C* ! super-pixel confident data  
		
C* decade 1  
#define O3_SNOBS_D1	1	
C* number of observations used for decade synthesis  
#define O3_STAU865_D1	2	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_STAU865_SMALL_D1	3	
C* mean of aerosol optical thickness from small model, for 865 nm  
#define O3_SANGST_D1	4	
C* mean of Angstrom coefficients  
#define O3_SANGST_SMALL_D1	5	
C* mean of Angstrom coefficients from small model 
#define O3_SINDEX_D1	6	
C* mean of aerosol index  
		
C* decade 2  
#define O3_SNOBS_D2	7	
C* number of observations used for decade synthesis  
#define O3_STAU865_D2	8	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_STAU865_SMALL_D2	9	
C* mean of aerosol optical thickness from small model, for 865 nm  
#define O3_SANGST_D2	10	
C* mean of Angstrom coefficients  
#define O3_SANGST_SMALL_D2	11	
C* mean of Angstrom coefficients from small model 
#define O3_SINDEX_D2	12	
C* mean of aerosol index  
		
C* decade 3  
#define O3_SNOBS_D3	13	
C* number of observations used for decade synthesis  
#define O3_STAU865_D3	14	
C* mean of aerosol optical thickness, for 865 nm  
#define O3_STAU865_SMALL_D3	15	
C* mean of aerosol optical thickness from small model, for 865 nm  
#define O3_SANGST_D3	16	
C* mean of Angstrom coefficients  
#define O3_SANGST_SMALL_D3	17	
C* mean of Angstrom coefficients from small model 
#define O3_SINDEX_D3	18	
C* mean of aerosol index  
                
C* month  
#define O3_SNOBS 	19	
C* number of observations used for monthly synthesis  
#define O3_SNOBS_OPT 	20	
C* number of observations used for monthly synthesis with optimal geom used
#define O3_STAU865	21	
C* mean of aerosol optical thickness, for 865 nm 
#define O3_STAU865_SMALL	22	
C* mean of aerosol optical thickness from small model, for 865 nm 
#define O3_SANGST	23	
C* mean of Angstrom coefficients 
#define O3_SANGST_SMALL	24	
C* mean of Angstrom coefficients from small model
#define O3_SINDEX	25	
C mean of aerosol index for monthly synthesis 
#define O3_STAU865_PP	26	
C* mean of tau from small model, for 865 nm, with optimal geom used 
#define O3_STAU865_GS	27	
C* mean of tau from spherical coarse mode, for 865 nm, with optimal geom used 
#define O3_STAU865_GNS	28	
C* mean of tau from non-spherical coarse mode, for 865 nm, with optimal geom used 
#define O3_STAU865_GC	29	
C* mean of tau from  coarse mode, for 865 nm, with optimal geom used 
#define O3_SID		30	
C* relative contribution of non spherical particles to O3_STAU865_GC 
#define O3_SQTAU865_1  	31	
C* aerosol optical thickness monthly quartile, for 865 nm 
#define O3_SQTAU865_2  	32	
C* aerosol optical thickness monthly quartile, for 865 nm 
#define O3_SQTAU865_3  	33	
C* aerosol optical thickness monthly quartile, for 865 nm 
#define O3_SQTAU865_4  	34	
C* aerosol optical thickness monthly quartile, for 865 nm 
#define O3_SQTAU865_5  	35	
C* aerosol optical thickness monthly quartile, for 865 nm 
#define O3_SQTAU865_SMALL_1 	36	
C* aerosol optical thickness monthly quartile from small model, for 865 nm 
#define O3_SQTAU865_SMALL_2 	37	
C* aerosol optical thickness monthly quartile from small model, for 865 nm 
#define O3_SQTAU865_SMALL_3 	38	
C* aerosol optical thickness monthly quartile from small model, for 865 nm 
#define O3_SQTAU865_SMALL_4 	39	
C* aerosol optical thickness monthly quartile from small model, for 865 nm 
#define O3_SQTAU865_SMALL_5 	40	
C* aerosol optical thickness monthly quartile from small model, for 865 nm 
#define O3_SANGST_1	41	
C* Angstrom coefficient monthly frequency 
#define O3_SANGST_2	42	
C* Angstrom coefficient monthly frequency 
#define O3_SANGST_3	43	
C* Angstrom coefficient monthly frequency 
#define O3_SANGST_4	44	
C* Angstrom coefficient monthly frequency 
#define O3_SANGST_SMALL_1	45	
C* Angstrom coefficient monthly frequency from small model
#define O3_SANGST_SMALL_2	46	
C* Angstrom coefficient monthly frequency from small model
#define O3_SANGST_SMALL_3	47	
C* Angstrom coefficient monthly frequency from small model
#define O3_SANGST_SMALL_4	48	
C* Angstrom coefficient monthly frequency from small model
#define O3_SINDEX_SMALL_1	49	
C* aerosols refractive index monthly frequency from small model
#define O3_SINDEX_SMALL_2	50	
C* aerosols refractive index monthly frequency from small model
#define O3_SINDEX_SMALL_3	51	
C* aerosols refractive index monthly frequency from small model
#define O3_SINDEX_BIG_1	52	
C* aerosols refractive index monthly frequency from big model
#define O3_SINDEX_BIG_2	53	
C* aerosols refractive index monthly frequency from big model
#define O3_SINDEX_BIG_3	54	
C* aerosols refractive index monthly frequency from big model
#define O3_SREFFRAD_1	55	
C* aerosols effective beam monthly frequency  
#define O3_SREFFRAD_2	56	
C* aerosols effective beam monthly frequency  
#define O3_SREFFRAD_3	57	
C* aerosols effective beam monthly frequency  
#define O3_SREFFRAD_4	58	
C* aerosols effective beam monthly frequency  
#define O3_SREFFRAD_SMALL_1	59	
C* aerosols effective beam monthly frequency from small model 
#define O3_SREFFRAD_SMALL_2	60	
C* aerosols effective beam monthly frequency from small model 
#define O3_SREFFRAD_SMALL_3	61	
C* aerosols effective beam monthly frequency from small model 
#define O3_SREFFRAD_SMALL_4	62	
C* aerosols effective beam monthly frequency from small model 


C*  LEVEL 2   
C*  Earth radiation and clouds  
C*----------------------------- 
#define RC_OPT		536870912   
C* OPTION for distribution  

#define R2B		56
#define R2_TAB_I	8192
#define R2_TAB_J	512
#define R2B_NB_PAR_DIR	10	
C* number of directional parameters  
#define R2_DQS		0	
C* ! super-pixel confidence data  
#define R2_H		1	
C* super-pixel UTC time : hour  
#define R2_MN		2	
C* super-pixel UTC time : minutes  
#define R2_NI		3	
C* number of viewing directions  
#define R2_NRNP		4	
C* number of viewing directions for Rayleigh    
C* number of elementary pixels 
#define R2_MINMAXGLIT	5	
C* number of the first direction in the glitter  
C* number of the last direction in the glitter  
#define R2_MU0		6	
C* cosine of solar zenithal angle for centralpixel  
#define R2_AVIS		7	
C* Mean of albedos calculated by pixel and by direction   
C* in visible wavelenght (670 nm over land or 865 over sea)   
#define R2_RSDAVIS	8	
C* Relative spatial dispertion of albedos calculated by pixel 
C* and by direction in visible wavelenght  
#define R2_RADAVIS	9	
C* Relative angular dispertion of albedos calculated by pixel 
C* and by direction in visible wavelenght  
#define R2_QAVIS	10	
C* Quality index dispertion of albedos calculated by pixel 
C* and by direction in visible wavelenght  
#define R2_ASCVIS	11	
C* albedo of the scene supposed as homgeneous in the visible  
C* wavelenght (670 nm over land or 865 over sea)  
#define R2_RADASCVIS	12	
C*Relative spatial dispertion of albedo of the scene supposed  
C* as homgeneous in the visible wavelenght (670 nm over land or 865 over sea)  	
#define R2_AVISCLEAR	13	
C* "estimated clear sky" albedo in visiblewavelenght  
C* (670 nm over land or 865 over sea)  
#define R2_ASW		14	
C* Albedo for lambda SW  
#define R2_ASWCLEAR	15	
C* "estimated clear sky" albedo for lambda SW  
#define R2_CN		16	
C* Cloud cover  
#define R2_CNPLUSMOINS	17	
C* Rate of indefinite pixels redistributed "clear sky"  
C* Rate of indefinite pixels redistributed "cloudy"  
#define R2_QCN		18	
C* Quality index of cloud cover  
#define R2_M_UH2O	19	
C* mean of apparent water vapor content on clear sky pixels  
#define R2_QH2O		20	
C* Apparent water vapour content on clear sky pixels  
#define R2_M_PCLOUD	21	
C* mean of corrected cloud O2 pressure (on cloudy pixels)  
#define R2_S_PCLOUD	22	
C* Angular deviation of  cloud O2 pressure  on cloudy pixels) 
#define R2_M_RAYL	23	
C* mean of cloud Rayleigh pressure (on cloudy pixels)  
#define R2_S_RAYL	24	
C* Angular deviation of cloud Rayleigh pressure (on cloudy pixels) 
#define R2_TAU		25	
C* Mean of optical thicness on cloudy pixels in visible wavelenght  
C* (670 nm over land or 865 over sea)  
#define R2_RSDTAU	26	
C* Relative standar deviation of optical thickness on cloudy pixels  
C* in visible wavelenght (670 nm over land or 865 over sea)   
#define R2_EPSTAU	27	
C* coefficient of heterogeneousness on clouy superpixels  
C* in visible wavelenght (670 nm over land or 865 over sea)   
#define R2_ASVIS	28	
C* Cloud spherical albedo in visible wavelenght 
C* (670 nm over land or 865 over sea)   
#define R2_INDPHASE	29
C* cloud phase quality index (liquid, ice, mixed, 
C* undetermined, non calculated with quality hierarchy 				
#define R2_IND_MICRO	30	
C* indicator of ice crystals type 
#define R2_STRAT670	31	
C* stratospheric aerosols optical thickness for 670 nm 
#define R2_UO3		32	
C* integrated contents of ozone 
#define R2_VVS		33	
C* surface wind speed 
#define R2_PHIVS	34	
C* surface wind direction 
#define R2_PSURF	35	
C* surface pressure  
#define R2_TK		36+R2_TAB_I + 10*R2_TAB_J	
C* temperature profile, estimated from Meteo data 
#define R2_UK		46+R2_TAB_I + 10*R2_TAB_J	
C* integrated water vapor profile, estimated from Meteo data 	
C* directional parameters  
#define R2_M_THV	56	
C* zenithal viewing angle for central pixel 
#define R2_M_AZ		57	
C* relative azimuth angle for central pixel 
#define R2_M_RVIS	58	
C* Mean of Np corrected reflectance in visible 
C* wavelenght (670 nm over land or 865 over sea)  
#define R2_M_AVIS	59	
C* Mean of Np albedo in visible wavelenght (670 nm over land or 865 over sea) 				
#define R2_M_RSW	60	
C* Mean of Np corrected reflectance for lambda SW
#define R2_M_ASW	61	
C* Mean of Np albedo  for lambda SW 				
#define R2_LPMOD865	62      
C* mean of Np polaried "modified" radiances for 865 nm
#define R2_PCTCLOUCLEAR	63	
C* ratio of cloud pixels before "indetermination" removal (10 classes) 
C* ratio of cloud sky pixels before "indetermination" removal (10 classes) 
#define R2_DCN		64	
C* directional cloud cover 
#define R2_SVIS		65	
C* Cloud spherical albedo in visible wavelenght (670 nm over land or 865 over sea) 

		
								


C*  LEVEL 3   
C*  Earth radiation and clouds  
C*----------------------------- 
#define R3B		61
#define R3_TAB_I	8192
#define R3_TAB_J	512
#define R3_NJ		0	
C* number of observation days  
#define R3_NORB_TOT	1	
C* number of orbits / super-pixel processed  
#define R3_NORB_SNOW	2	
C* number of orbits / snow-ice risk  
#define R3_NORB_CLEAR	3	
C* number of orbits/cloud cover is zero  
#define R3_NORB_CN	4	
C* number of orbits/cloud cover not zero  
#define R3_NORB_TAU	5	
C* number of orbits/albedo determined  
#define R3_NORB_PCLOUD	6	
C* number of orbits/corrected cloud O2 pressure determined  
#define R3_NORB_PRAY	7	
C* number of orbits/cloud Rayleigh pressure determined  
#define R3_NORB_PHASE	8	
C* number of orbits/phase determined  
#define R3_NORB_UH2O	9	
C* number of orbits/UH2O determined  
#define R3_M_MU0	10	
C* monthly mean of zenithal solar cosines MUs  
#define R3_M_AVIS	11	
C* monthly mean of albedos in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_S_AVIS	12	
C* temporal standard deviation of albedos in visible wavelenght  
C* (670nm over land or 865 over sea)   
#define R3_M_AVISCLEAR	13	
C* monthly mean of " clear sky" albedos in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_S_AVISCLEAR	14	
C* temporal standard deviation of " clear sky" albedos in visible  
C* wavelenght (670nm over land or 865 over sea)   								
#define R3_M_AVISMCLEAR	15	
C* monthly mean of " estimated clear sky" albedos in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_M_ASW	16	
C* monthly mean of albedos , for SW  
#define R3_S_ASW	17	
C* temporal standard deviation  of albedos , for SW  
#define R3_M_ASWCLEAR	18	
C* monthly mean of" clear sky"  albedos , for SW  
#define R3_S_ASWCLEAR	19	
C* temporal standard deviation  of" clear sky" albedos , for SW  
#define R3_M_ASWMCLEAR	20	
C* monthly mean of" estimated clear sky"  albedos , for SW  
#define R3_M_FINC	21	
C* monthly mean the incidental diurnal flux  
#define R3_M_FREFL	22	
C* monthly mean the reflected diurnal flux  
#define R3_M_FREFLCLEAR	23	
C* monthly mean the "clear sky" reflected diurnal flux  
#define R3_M_CN		24	
C* monthly mean of cloud cover  
#define R3_S_CN		25	
C* temporal standard deviation of cloud cover  
#define R3_M_CLEARCLOUD 26	
C* monthly rate of indefinite pixels redistributed  
C* "clear sky"  outside glitter 
C* monthly rate of indefinite pixels redistributed 
C* "cloudy" outside glitter 
#define R3_M_H2O	27	
C* monthly mean of apparent water vapor content on clear-sky pixels  
#define R3_S_H2O	28	
C* temporal standard deviation of apparent water vapor content on clear-sky pixels  
#define R3_M_PCLOUD	29	
C* monthly mean of corrected cloud O2 pressure,  
C* balanced with CN (on cloudy pixels) 
#define R3_S_PCLOUD	30	
C* temporale standard deviation of corrected cloud O2 pressure,  
C* balanced with CN (on cloudy pixels) 
#define R3_M_PRAY	31	
C* monthly mean of cloud Rayleigh pressure (on cloudy pixels)  
#define R3_S_PRAY	32 	
C* temporal standard deviation of cloud Rayleigh pressure (on cloudy pixels) 
#define R3_M_TAU	33	
C* mean of cloud optical thickness in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_SR_TAU	34	
C* relative temporal standard deviation of cloud optical thickness in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_M_TAULIQ	35	
C* mean of liquid cloud optical thickness in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_M_TAUICE	36	
C* mean of ice cloud optical thickness in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_M_TAUMIX	37	
C* mean of mixed phase cloud optical thickness in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_M_ASVIS	38	
C* monthly mean of spherical albedos in visible wavelenght  
C* (670nm over land or 865 over sea)  
#define R3_S_ASVIS	39
C* temporal standard deviation of spherical albedos in visible wavelenght  
C* (670nm over land or 865 over sea)   
#define R3_FRE_PHASE	40+R3_TAB_I + 4*R3_TAB_J	
C* cloud phase distribution (cover percentage in 4 classes)  
#define R3_FRE_CRIS	44+R3_TAB_I + 7*R3_TAB_J	
C* ice crystal types distribution (cover percentage in 7 classes)  

 

#endif
