/***************************************************************************
 *   Copyright (C) 2005 by Nicolas PASCAL                                  *
 *   nicolas.pascal@icare.univ-lille1.fr                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "file_tools.h"

bool file_can_be_opened_as(const string & path_and_filename , const string &mode ) {
    // check validity of the open mode
    if ( mode != "r" && mode != "w" && mode != "rw") {
        cerr<<"Invalid opening mode "<<mode<<" in "<<__FILE__<<" at "<<__LINE__<<endl;
        return false;
    }

    bool is_file_valid=false;
    FILE* fp=NULL;
    try { // for failsafe directory opening
      fp = fopen(path_and_filename.c_str(),mode.c_str());
      if (fp!=NULL) {
        is_file_valid=true;
        fclose(fp);
      }
    } catch(...) {
      fclose(fp);
      throw;
    }
    return is_file_valid;
};
bool exists_file(const string &path_and_filename ) {
    return file_can_be_opened_as(path_and_filename,"r");
};
bool exists_dir(const string &dir_path) {
  bool is_dir_valid=false;
  DIR* dir=NULL;
  try { // for failsafe directory opening
    dir=opendir(dir_path.c_str());
    if (dir!=NULL) {
      is_dir_valid=true;
      closedir(dir);
    }
  } catch(...) {
    closedir(dir);
    throw;
  }
  return is_dir_valid;
}
const string get_filename(const string &path_and_filename ) {
    if (&path_and_filename == NULL)
        return string("");

    string::const_iterator it = path_and_filename.end();
    string::const_iterator begin = path_and_filename.begin();
    while (it != begin && *it!='/')
        --it; // searching for the slash : portability problem issue here

    if (*it=='/') {
        return string(it+1,path_and_filename.end());
    } else
        return string("");
};
const string get_directory(const string &path_and_filename ) {
    int last_slash_pos = path_and_filename.find_last_of('/');
    if (last_slash_pos==(int)string::npos)
        // '/' not found
        return "";
    else
        return path_and_filename.substr(0,last_slash_pos+1);
};
const string basename( const string & path_and_filename )
{
    return get_tail(path_and_filename);
}
const string get_tail( const string & path_and_filename )
{
    int last_slash_pos = path_and_filename.find_last_of('/');
    if (last_slash_pos==(int)string::npos)
        // '/' not found
        return path_and_filename;
    else
        return path_and_filename.substr(last_slash_pos+1,path_and_filename.size()-last_slash_pos);
}
const string get_tmp_dir()
{
    // try to find a temporary dir : can depends of the system
    FILE *result = popen("echo $TMPDIR","r");
    string s_result = "";
    if ( result != NULL ) {
        char buf[256];
        if (fgets(buf, sizeof buf, result)) {
            // temp file can be created
            s_result=buf;
            if (!s_result.empty())
                // remove trailing '\n'
                s_result.erase(s_result.end()-1);
            pclose(result);
        } else {
            pclose(result);
            bad_tmp_dir e = bad_tmp_dir();
            throw e;
        }
    }
    return s_result;
}
