/***************************************************************************
 *   Copyright (C) 2005 by Nicolas PASCAL                                  *
 *   nicolas.pascal@icare.univ-lille1.fr                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILES_TOOLS_H
#define FILES_TOOLS_H

#include <string>
#include <iostream>
#include <cstdio>
#include <sys/types.h>
#include <dirent.h>
using namespace std;

class bad_tmp_dir {
public:
    bad_tmp_dir() {};
    ~bad_tmp_dir() {};
    void what() const {
        cout <<"bad_tmp_dir : can't find the temporary directory. Review the get_temp_dir function in file_tools.cpp"<<endl;
    };
};

/**
Few files' manipulation utils
@author Nicolas PASCAL
*/

/**
test if the file can be opened in the given mode.
@param path_and_filename the file's path+"/"+filename
@param mode the opening mode
@return true if the file can be opened in the given mode
*/
bool file_can_be_opened_as(const string & path_and_filename = "", const string &mode = "r");

/**
test the existence of a file.
@param path_and_filename the file's path+"/"+filename
@return true if the file exists
*/
bool exists_file(const string &path_and_filename = "");
/**
test the existence of a directory. In test if the directory is accessible
@param dir_path the directory's path
@return true if the file exists
*/
bool exists_dir(const string &dir_path);
/**
* Extract a path from a full filename.
* If @a filename doesn't contain a '/' char, the method will return it unchanged
* @param path_and_filename the file's path+"/"+filename
* @return the directory path with ending '/' char.
*/
const string get_directory(const string &path_and_filename = "");
/**
 * @brief extract the tail of a filename (the name without its directory path)
 * If @a filename doesn't contain a '/' char, the method will return it unchanged
 * @param filename the full filename (directory path + name)
 * @return the file name's tail (without starting '/' char).
 */
const string get_tail(const string &filename = "");
/**
 * @brief extract the tail of a filename (the name without its directory path)
 * If @a filename doesn't contain a '/' char, the method will return it unchanged
 * @param filename the full filename (directory path + name)
 * @return the file name's tail (without starting '/' char).
 */
const string basename(const string &filename = "");
/**
@brief find a temporary directory
The return string doesn't contains the trailing '/' char
At this time, it only tries to read the TMPDIR variable. This method can be enhanced to be more portable, like the stdio tempnam() fucnction.
Throws a bad_temp_dir exception if it can't be found.
*/
const string get_tmp_dir();
#endif


