//
//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "hdf5filedata.h"
#include "H5Fpublic.h"
#include "H5public.h"
#include "H5Spublic.h"
#include "H5Apublic.h"
#include "H5LTpublic.h"
#include "Hdf_sds.hpp"

HDF5FileData::HDF5FileData(const string &filename, const string &mode)
:FileData(filename,mode),FileDataReader(filename,mode)
{
    name = filename;
    file_id=-1;
    read_dataset_done = false;

    read_all_dataset();

    string date = get_values_attr("RANGEBEGINNINGDATE");

    int year = MyTools::to_num<int>(date.substr(0,4));
    int month = MyTools::to_num<int>(date.substr(5,2));
    int day = MyTools::to_num<int>(date.substr(8,2));

    string time = get_values_attr("RANGEBEGINNINGTIME");

    int hour = MyTools::to_num<int>(time.substr(0,2));
    int minut = MyTools::to_num<int>(time.substr(3,2));
    int seconds = MyTools::to_num<int>(time.substr(6,2));

    Date bdate(year,month,day,hour,minut,seconds);

    date = get_values_attr("RANGEENDINGDATE");

    year = MyTools::to_num<int>(date.substr(0,4));
    month = MyTools::to_num<int>(date.substr(5,2));
    day = MyTools::to_num<int>(date.substr(8,2));

    time = get_values_attr("RANGEENDINGTIME");

    hour = MyTools::to_num<int>(time.substr(0,2));
    minut = MyTools::to_num<int>(time.substr(3,2));
    seconds = MyTools::to_num<int>(time.substr(6,2));

    Date edate(year,month,day,hour,minut,seconds);

    this->set_date(bdate);
    this->time_coverage= edate.get_epoch_time()-bdate.get_epoch_time();
}

HDF5FileData::~HDF5FileData()
{
    for(uint i=0; i<dataset_vect.size(); i++) H5Dclose(dataset_map[dataset_vect[i]]);
    for(uint i=0; i<attributes.size(); i++) H5Dclose(attributes[i]);
    dataset_map.clear();
    dataset_vect.clear();
    read_dataset_done = false;
}

void HDF5FileData::open_data_file()
{
    file_id=H5Fopen(name.c_str(),H5F_ACC_RDONLY,H5P_DEFAULT);
    if(!read_dataset_done) read_all_dataset();
}

void HDF5FileData::close_data_file()
{
    if(file_id==-1) return;
    H5Fclose(file_id);
    file_id=-1;
    for(uint i=0; i<dataset_vect.size(); i++) H5Dclose(dataset_map[dataset_vect[i]]);
    dataset_map.clear();
    dataset_vect.clear();
    read_dataset_done = false;
}


void HDF5FileData::free_read_write_allocations( const bool *are_limits_initialized, int *start, int *stride, int *edges ) {
    if ( are_limits_initialized[0])
        delete[] start;
    if ( are_limits_initialized[1])
        delete[] stride;
    if ( are_limits_initialized[2])
        delete[] edges;
};
const bool HDF5FileData::check_read_write_limits(const char* sds_name, int *start, int *stride, int *edges, const int rank) {
    assert(start!=NULL);
    assert(edges!=NULL);
    vector<int> dim=this->get_dataset_dimension(sds_name);
    for ( int i = 0 ; i<rank ; i++ ) {
        if (start[i] < 0 || start[i] > (dim[i]-1)) {
            bad_index e(start[i],(dim[i]-1));
            throw e;
            return false;
        }

        // test the selection length
        int stride_val;
        if ( stride!=NULL )
            stride_val=stride[i];
        else
            stride_val=1;

        if ( (start[i]+(edges[i]*stride_val))> dim[i] ) {
            bad_index e(start[i]+(edges[i]*stride_val),dim[i]-1);
            throw e;
            return false;
        }
    }
    return true;
};
void HDF5FileData::init_read_write_null_input_param(const char* sds_name, int *&start, int *&stride, int *&edges, int &rank, bool * initialized_values) {
    initialized_values[0] = false;
    initialized_values[1] = false;
    initialized_values[2] = false;

    vector<int> dim=this->get_dataset_dimension(sds_name);

    // if rank is not defined, set to the sds one
    if (rank==-1)
        rank = dim.size();
    // if start is not defined, allocate it with a rank size, and set its values to 0
    if ( start == NULL ) {
        start = new int[rank];
        initialized_values[0] = true;
        for ( int i = 0;i<rank;++i ) {
            start[i]=0;
        }
    }
    // if edges is not defined, allocate it with a rank size, and set its values using the sds dimensions
    if ( edges == NULL ) {
        edges = new int[rank];
        initialized_values[2] = true;
        for ( int i = 0;i<rank;++i ) {
            edges[i]=dim[i];
        }
    }

    if ( stride == NULL ) {
        stride = new int[rank];
        initialized_values[1] = true;
        for ( int i = 0;i<rank;++i ) {
            stride[i]=1;
        }
    }
};
void* HDF5FileData::read_data( void* data, const char * sds_name, int * start, int * stride, int * edges, int rank ) {
    // TODO set edges to return the size of the read array
    bool are_limits_initialized[3]; // check if start, stride or edges had been initialized in this method
//     bool hdf_file_already_loaded = is_hdf_file_loaded();
//     if (!hdf_file_already_loaded)
//         load_hdf_file();
    // --- initialize NULL input parametres ---
    init_read_write_null_input_param(sds_name, start, stride, edges,rank, are_limits_initialized);
    // check the selection limits
    check_read_write_limits(sds_name, start,stride,edges,rank); // throw bad_index in case of problem
    // read the desired value, in the good format
    int data_size = 1;
    vector<int> vdim=this->get_dataset_dimension(sds_name);
    hsize_t *dim = new hsize_t[rank];
    for (int i = 0 ; i<rank ; ++i){
        data_size*=edges[i];
        dim[i]=vdim[i];
    }

    vector<int> size=this->get_dataset_dimension(sds_name);

    if(!read_dataset_done) read_all_dataset();
    hid_t dataset_id = dataset_map[sds_name];
    hid_t type_id=H5Dget_type(dataset_id);
//     hid_t dsp_id = H5Screate(H5S_SIMPLE);
//     H5Sset_extent_simple(dsp_id, rank, dim, dim  );

    hsize_t *start_t= new hsize_t[rank];
    hsize_t *stride_t= new hsize_t[rank];
    hsize_t *edges_t= new hsize_t[rank];
    hsize_t *dim_t= new hsize_t[rank];
    for(int i=0; i<rank; i++){
        start_t[i] = (hsize_t)start[i];
        stride_t[i] = (hsize_t)stride[i];
        edges_t[i] = (hsize_t)edges[i];
        dim_t[i]= stride[i]>1 ? (edges[i]+1)/stride[i] : edges[i]/stride[i];
    }

    hid_t dsp_id = H5Dget_space(dataset_id);
    H5Sselect_hyperslab(dsp_id, H5S_SELECT_SET, start_t, stride_t, edges_t, NULL  );

    hid_t memspace_id = H5Screate_simple (rank, dim_t, NULL);

    delete start_t;
    delete stride_t;
    delete edges_t;

    // data allocation hasn't been done by the caller
    if (data==NULL) {
        // read the size of the sds data
        size_t data_type_size = H5Tget_size( type_id  );
        data = static_cast<void*>(malloc(data_type_size*data_size));
    }

    H5Dread(dataset_id, type_id, memspace_id, dsp_id, H5P_DEFAULT, data );

    // free the ressources allocated by this method
    free_read_write_allocations(are_limits_initialized, start, stride, edges);
//     if (!hdf_file_already_loaded)
//         free_hdf_file();

    return data;
}

vector<int> HDF5FileData::get_dataset_dimension(const string &sds_name)
{

    if(!read_dataset_done) read_all_dataset();
    hid_t dataset = dataset_map[sds_name];
    hid_t dsp_id = H5Dget_space(dataset);
    int n = H5Sget_simple_extent_ndims(dsp_id);

    hsize_t *dims = new hsize_t[n];
    hsize_t *maxdims = new hsize_t[n];
    vector<int> v;

    H5Sget_simple_extent_dims(dsp_id, dims, maxdims);
    for(int i=0; i<n; i++) v.push_back((int) dims[i]);

    return v;
}


void HDF5FileData::get_dataset_fill_value(const string &sds_name, void* fill_value)
{
    if(!read_dataset_done) read_all_dataset();
    hid_t dataset_id = dataset_map[sds_name];
    hid_t type_id=H5Dget_type(dataset_id);
    hid_t plist = H5Dget_create_plist(dataset_id);
    H5Pget_fill_value(plist, type_id, fill_value );
}

int HDF5FileData::get_n_dataset()
{
    if(!read_dataset_done) read_all_dataset();
    return dataset_vect.size();
};

string HDF5FileData::get_dataset_name(int i)
{
    if(!read_dataset_done) read_all_dataset();
    return dataset_vect[i];
};

int HDF5FileData::get_dataset_data_type(string sds_name)
{
    if(!read_dataset_done) read_all_dataset();
    hid_t dataset_id = dataset_map[sds_name];
    hid_t type_id=H5Dget_type(dataset_id);
    hid_t native_type_id=H5Tget_native_type(type_id,H5T_DIR_ASCEND);

    if(H5Tequal(native_type_id,H5T_NATIVE_CHAR)) {return DFNT_INT8;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_SHORT)) {return DFNT_INT16;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_INT)) {return DFNT_INT32;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_LONG)) {return DFNT_INT64;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_LLONG)) {return DFNT_INT32;}

    else if(H5Tequal(native_type_id,H5T_NATIVE_UCHAR)) {return DFNT_UINT8;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_USHORT)) {return DFNT_UINT16;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_UINT)) {return DFNT_UINT32;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_ULONG)) {return DFNT_UINT64;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_ULLONG)) {return DFNT_UINT32;}

    else if(H5Tequal(native_type_id,H5T_NATIVE_FLOAT)) {return DFNT_FLOAT32;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_DOUBLE)) {return DFNT_FLOAT64;}
    else if(H5Tequal(native_type_id,H5T_NATIVE_LDOUBLE)) {return DFNT_FLOAT64;}
    else return native_type_id;
}

string HDF5FileData::get_values_attr_dataset(string sds_name,string attr_name)
{
    if(!read_dataset_done) read_all_dataset();
    hid_t dataset_id = dataset_map[sds_name];
    hid_t attr_id=H5Aopen_name(dataset_id, attr_name.c_str());
    hid_t type_id = H5Aget_type(attr_id);
    //hsize_t size=H5Aget_storage_size(attr_id);
    char buf[100];
    H5Aread(attr_id, type_id, buf  );
    return string(buf);
}

bool HDF5FileData::has_attr_dataset(string sds_name,string attr_name)
{
    if(!read_dataset_done) read_all_dataset();
    hid_t dataset_id = dataset_map[sds_name];
    return (bool)H5Aexists( dataset_id, attr_name.c_str()  );
}

string HDF5FileData::get_values_attr(string attr_name)
{
    return attribute[attr_name];
}

bool HDF5FileData::has_attr(string attr_name)
{
    return attribute.count(attr_name);
}

void HDF5FileData::read_all_dataset()
{
    int n;

    bool opened = file_id>=0;

    if(!opened) open_data_file();

    if(read_dataset_done) return;

    char *gp_name = new char[100];

    hsize_t*  num_obj= new hsize_t();

    hid_t gp_id;
    hid_t gp_id1;
    hid_t dsp_id;

    vector<hid_t> v;

    v.push_back(file_id);

    while(!v.empty()){

       gp_id = v.back();

       v.pop_back();

        H5Gget_num_objs(gp_id, num_obj);

        n = *num_obj;

        for(int i=0; i<n; i++){
            int type=H5Gget_objtype_by_idx(gp_id, i);
            if(type == H5G_GROUP){
                H5Gget_objname_by_idx(gp_id, i, gp_name, 100);
                gp_id1 = H5Gopen(gp_id,gp_name,NULL);
                v.push_back(gp_id1);
            }else if(type == H5G_DATASET){
                H5Gget_objname_by_idx(gp_id, i, gp_name, 100);
                gp_id1 = H5Dopen(gp_id,gp_name,NULL);
                dsp_id = H5Dget_space(gp_id1);
                if(H5Sget_simple_extent_ndims(dsp_id)>0) {
                    dataset_map[string(gp_name)]=gp_id1;
                    dataset_vect.push_back(string(gp_name));
                }else{
                    attributes.push_back(gp_id1);
                }
            }
        }

        if(gp_id != file_id) H5Gclose(gp_id);
    }

    read_metadata();

    read_dataset_done = true;

    if(!opened) close_data_file();
}


void HDF5FileData::read_metadata(){

    for(uint i=0; i<attributes.size(); i++){
        hid_t dataset_id = attributes[i];
        hid_t type_id = H5Dget_type(dataset_id);
        hid_t dsp_id = H5Dget_space(dataset_id  );
        size_t size = H5Tget_size(type_id);
        char data[size];
        H5Dread(dataset_id, type_id, H5S_ALL, dsp_id, H5P_DEFAULT, data );
        string text = string(data);
        vector<string> lines;
        split(text, lines, "\n");

        string tag;
        string value;
        string object;

        for(uint j=0; j<lines.size(); j++){
            read_line(lines[j],tag,value);
            if(tag=="OBJECT"){
                object = value;
            }else if(tag=="END_OBJECT"){
                object = "";
            }else if(tag == "VALUE"){
                if(object != "") {
                    attribute[object]=value;
                }
            }
        }

    }
}



void HDF5FileData::read_line(string line, string &tag, string &value){
    int index = line.find_first_of('=');
    if(index <0){
        tag="";
        value="";
        return;
    }

    tag = line.substr(0,index);
    trim(tag);
    value = line.substr(index+1);
    trim(value);
}

void HDF5FileData::trim(string &s){

    string delimiters = " \v\t\r\n\"";

    if (&s!=NULL && !s.empty()){
        std::size_t start = s.find_first_of(delimiters,0); // min space range index
        std::size_t length = s.find_first_not_of(delimiters,start)-start;// length of space range
        while (start!=std::string::npos) {
            s.erase(start,length);
            start = s.find_first_of(delimiters,start);
            length = s.find_first_not_of(delimiters,start)-start;
        }
    }
}

void HDF5FileData::split(const string& str,
                      vector<string>& tokens,
                      const string& delimiters)
{
    int pos1=0;
    int pos2=str.find_first_of(delimiters,pos1+1);
    string line;

    while(pos2!=-1){
        line = str.substr(pos1+1,pos2-1-pos1);
        tokens.push_back(line);
        pos1=pos2;
        pos2=str.find_first_of(delimiters,pos1+1);
    }
    line = str.substr(pos1+1,str.length()-pos1);
    tokens.push_back(line);
}

void HDF5FileData::get_fillValue(const string &sds_name, void *fillValue)
{
    if(!read_dataset_done) read_all_dataset();
    hid_t dataset_id = dataset_map[sds_name];
    hid_t type_id=H5Dget_type(dataset_id);
    hid_t plist = H5Dget_create_plist(dataset_id);
    H5Pget_fill_value(plist, type_id, fillValue );
};


void HDF5FileData::get_scaling(const string &sds_name, float64 &scale, float64 &offset)
{
    if(!read_dataset_done) read_all_dataset();
//     hid_t dataset_id = dataset_map[sds_name];
//     hid_t type_id=H5Dget_type(dataset_id);
//     hid_t plist = H5Dget_create_plist(dataset_id);
    //TODO adapte code
        scale=1.0;
        offset=0.0;
};

