/***************************************************************************
 *   Copyright (C) 2006 by Nicolas PASCAL   *
 *   pascal@icare-pc12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "logfile.h"

Logfile::Logfile( const string &filename ) {
  open(filename);
}

void Logfile::write( const string &msg, const string file, const int line ) {
  string _msg("");

  if (!file.empty())
    _msg=file;
  if (line!=-1)
    _msg+="("+MyTools::to_string(line)+") ";
  if ( ! _msg.empty() )
    _msg+="\n";
  _msg+=msg;
  if (this->is_opened())
    fp<<_msg<<endl;
  else {
    file_not_opened e(filename);
    throw e;
  }
}

Logfile::~Logfile( )
{
  close();
}

void Logfile::close( )
{
  if (this->is_opened())
    fp.close();
}

Logfile::Logfile( )
{
  filename="";
}

void Logfile::open( const string & filename )
{
  this->filename=filename;
  try {
    // try to open the file in write mode
    fp.open(filename.c_str());
    if (fp.fail()) {
      bad_file_opening_mode e(filename,"w");
      throw e;
    }
  } catch(...) {
    close();
    throw;
  }
}

const bool Logfile::is_opened() {
    return (fp.is_open());
}
