/***************************************************************************
 *   Copyright (C) 2006 by Nicolas PASCAL   *
 *   pascal@icare-pc12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TOOLS_EXCEPTIONS_H
#define TOOLS_EXCEPTIONS_H

#include <exception>
#include <string>
#include <sstream>
#include "tools.h"
using namespace std;
/**
Some exceptions issued of tools.cpp
*/

/**
 * @class InterpolationError -
 */
class InterpolationError : public exception {
public:
    /**
     * @brief constructor
     * @param src_file name of the source code file where the error occurs (ie __FILE__)
     * @param src_line number of the line of the source code file where the error occurs (ie __LINE__)
     * @param msg an error message
     */
    InterpolationError(const string &src_file, const int src_line, const string &msg): src_file(src_file),src_line(src_line),msg(msg){};
    ~InterpolationError() throw() {}
    ;
    virtual const char* what() const throw() {
      std::ostringstream oss;
      oss<<src_file<<"("<<src_line<<")"<<" InterpolationError : "<<msg<<endl;
      return oss.str().c_str();
    };
private:
    string src_file;
    int src_line;
    string msg;
};

/**
  * Thrown when trying to access an invalid index in a sequence
  */
class out_of_bound : public std::exception {
public:
    out_of_bound(const int index, const int size):index(index),size(size) {};
    ~out_of_bound() throw() {}
    ;
    virtual const char* what() const throw() {
      std::ostringstream oss;
      oss<<"out_of_bound : the index "<<index<<" is out of bound. It must be in the range [0,"<<size<<"["<<std::endl;
      return oss.str().c_str();
    };
private:
    int index;
    int size;
};

#endif
