/***************************************************************************
 *   Copyright (C) 2009 by Nicolas PASCAL   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "modisfiledata.h"

int main(int argc, char *argv[]) {
    int i_pix_1km [2], i_pix_5km [2];
    fill_n ( i_pix_1km, 2, -1 );
    fill_n ( i_pix_5km, 2, -1 );

    float32 lat, lon, lat_1km, lon_1km;

    MODISFileData fd ( "/DATA/LIENS/MODIS/MYD06_L2/2009/2009_01_01/MYD06_L2.A2009001.0420.005.2009047173755.hdf" );

    /* print out some informations */
    cout<<"Product "<<fd.get_product()<<endl;
    cout<<"Resolution "<<fd.get_resolution()<<endl;
    cout<<"Radix "<<fd.get_radix()<<endl;
    cout<<"Date "<<fd.get_date()<<endl;

    fd.load_geolocation_data();

    /* compute 1km pixels geolocation */
    cout << "compute 1km pixels geolocation ..." << endl;
    int sz_y_1km = fd.get_y_size_1km();
    int sz_x_1km = fd.get_x_size_1km();
    int sz_y_5km = fd.get_y_size_5km();
    int sz_x_5km = fd.get_x_size_5km();
    printf ( "sz_5km = [%d, %d] sz_1km=[%d, %d]\n", sz_y_5km, sz_x_5km, sz_y_1km, sz_x_1km );

    float32 * lat_data_1km = new float32 [ sz_y_1km * sz_x_1km ];
    float32 * lon_data_1km = new float32 [ sz_y_1km * sz_x_1km ];
    int i_x, i_y, i_y_start;
    for ( i_y = 0 ; i_y < sz_y_1km ; ++i_y )
    {
        if ( ( i_y % 500 ) == 0 )
            cout << "Scan line " << i_y << "/" << sz_y_1km << endl;
        i_y_start = i_y * sz_x_1km;
        for ( i_x = 0 ; i_x < sz_x_1km ; ++i_x )
        {
            fd.get_1km_pix_pos(i_y, i_x, lat, lon );
            lat_data_1km [ i_y_start + i_x ] = lat;
            lon_data_1km [ i_y_start + i_x ] = lon;
        }
    }
    cout << "... done" << endl;

//     // dump geolocation buffers
//     cout << "Dump geolocation datasets ..." << endl;
//     FILE * pf;
//     pf = fopen ("/home/pascal/depot/modis_geolocation/lat_data_1km.dat","wb");
//     if ( pf != NULL )
//     {
//         fwrite ( lat_data_1km , sizeof ( float32 ) , ( sz_y_1km * sz_x_1km ) , pf );
//         fclose ( pf );
//     }
//     delete [] lat_data_1km, lat_data_1km = NULL;
//
//     pf = fopen ("/home/pascal/depot/modis_geolocation/lon_data_1km.dat","wb");
//     if ( pf != NULL )
//     {
//         fwrite ( lon_data_1km , sizeof ( float32 ) , ( sz_y_1km * sz_x_1km ) , pf );
//         fclose ( pf );
//     }
//     delete [] lon_data_1km, lon_data_1km = NULL;
//     cout << "... done" << endl;

    /* test 5km pixel footprint search */

    /* coincidence search */
    // pixel out of granule
    lat = -72.49, lon = -50.06;
    fd.get_index ( lat, lon, -1., i_pix_5km, 0.2 );
    printf ( "P (%f,%f)\n i_5km [ %d,%d ]\n", lat, lon, i_pix_5km [0], i_pix_5km [1] );
    fd.get_5km_to_1km_pixel ( lat, lon, i_pix_5km, i_pix_1km );
    printf ( "i_1km [ %d,%d ]\n",i_pix_1km [0], i_pix_1km [1] );

    // pixel in granule
    lat = -73.91, lon = 170.53;
    fd.get_index ( lat, lon, -1., i_pix_5km, 0.2 );
    printf ( "P (%f,%f)\n i_5km [ %d,%d ]\n", lat, lon, i_pix_5km [0], i_pix_5km [1] );
    fd.get_5km_to_1km_pixel ( lat, lon, i_pix_5km, i_pix_1km );
    fd.get_1km_pix_pos ( i_pix_1km[0], i_pix_1km[1], lat_1km, lon_1km );
    printf ( "i_1km [ %d,%d ]\tcoord_1km (%f, %f)\tdist=%f\n",i_pix_1km [0], i_pix_1km [1], lat_1km, lon_1km, Geocentric::get_haversine_dist( lat, lon, lat_1km, lon_1km ) );
//
//     // pixel in granule, near change date meridian
    lat = -61.5, lon = - 179.95;
    fd.get_index ( lat, lon, -1., i_pix_5km, 0.2 );
    printf ( "P (%f,%f)\n i_5km [ %d,%d ] dist_5km=%f\n", lat, lon, i_pix_5km [0], i_pix_5km [1],
    Geocentric::get_haversine_dist( lat, lon,
                        fd.get_lat_data()[i_pix_5km [0] * 270 + i_pix_5km [1]],
                        fd.get_lon_data()[i_pix_5km [0] * 270 + i_pix_5km [1]] ) );
    fd.get_5km_to_1km_pixel ( lat, lon, i_pix_5km, i_pix_1km );
    fd.get_1km_pix_pos ( i_pix_1km[0], i_pix_1km[1], lat_1km, lon_1km );
    printf ( "i_1km [ %d,%d ]\tcoord_1km (%f, %f)\tdist=%f\n",i_pix_1km [0], i_pix_1km [1], lat_1km, lon_1km, Geocentric::get_haversine_dist( lat, lon, lat_1km, lon_1km ) );

    lat = -81.40, lon = 165.30;
    fd.get_index ( lat, lon, -1., i_pix_5km, 0.2 );
    printf ( "P (%f,%f)\n i_5km [ %d,%d ] dist_5km=%f\n", lat, lon, i_pix_5km [0], i_pix_5km [1],
    Geocentric::get_haversine_dist( lat, lon,
                        fd.get_lat_data()[i_pix_5km [0] * 270 + i_pix_5km [1]],
                        fd.get_lon_data()[i_pix_5km [0] * 270 + i_pix_5km [1]] ) );
    fd.get_5km_to_1km_pixel ( lat, lon, i_pix_5km, i_pix_1km );
    fd.get_1km_pix_pos ( i_pix_1km[0], i_pix_1km[1], lat_1km, lon_1km );
    printf ( "i_1km [ %d,%d ]\tcoord_1km (%f, %f)\tdist=%f\n",i_pix_1km [0], i_pix_1km [1], lat_1km, lon_1km, Geocentric::get_haversine_dist( lat, lon, lat_1km, lon_1km ) );

    fd.free_geolocation_data();
    fflush ( stdout );

    return 0;
}
