/***************************************************************************
 *   Copyright (C) 2006 by Nicolas PASCAL   *
 *   pascal@icare-pc12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "parasolfiledata.h"

int main(int argc, char *argv[]) {
    unsigned short nb_bytes;
    double slope,offset;
    bool show_val_display=true;
    string filename("/DATA/LIENS/PARASOL/OC2/2009/2009_01_01/P3L2TOGC094003KL");
    PARASOLFileData pfd(filename);
    int nb_data=pfd.get_nb_data();

    // ***** test some attributes building *****
    cout<<"-------------------------------------------------------"<<endl;
    cout<<"Instrument "<<pfd.get_instrument()<<endl;
    cout<<"Level "<<pfd.get_level()<<endl;
    cout<<"Processing line "<<pfd.get_processing_line_string()<<endl;
    cout<<"Product "<<pfd.get_product()<<endl;
    cout<<"Orbit cycle number "<<pfd.get_orbit_cycle_nb()<<endl;
    cout<<"Orbit number "<<pfd.get_orbit_nb()<<endl;
    cout<<"Reprocessing number "<<pfd.get_reprocessing_nb()<<endl;
    cout<<"Radix "<<pfd.get_radix()<<endl;
    cout<<"Software Version Number "<<pfd.get_software_version_number()<<endl;

    // ***** test geolocation data reading *****
    /*
    Init the geolocation data : it loads the lat,lon indexes in memory. To mandatory, but fasten methods that map lat,lon indexes to a record index like get_index(lat,lon,rec_idx).
    */
    pfd.load_geolocation_data();

    string param="tau_agns865";
    cout<<"---------- Extract Parameter "<<param<<" ----------"<<endl;
    pfd.get_scaling(param,slope,offset,nb_bytes);
    cout<<"Scale : "<<param<<" "<<nb_bytes<<" "<<slope<<" "<<offset<<endl;
    cout<<"---------------------------------------------------"<<endl;

    uint16* count_data  = static_cast<uint16 *>(pfd.read_count_data(NULL,param.c_str()));
    double* scaled_data = static_cast<double *>(pfd.read_scaled_data(NULL,param.c_str()));
    if (count_data!=NULL && scaled_data!=NULL) {
        if (show_val_display) {
            cout<<"Record\tCount\tScaled"<<endl;
            for (int i = 0 ; i<nb_data;++i) {
                cout<<i<<"\t"<<(int)count_data[i]<<"\t"<<scaled_data[i]<<endl;
            }
        }
    }
    delete[] count_data;
    count_data=NULL;
    delete[] scaled_data;
    scaled_data=NULL;


    return 0;
}
