#!/bin/bash -f

##############################################################################
#                     filedata compilation environment
#
# Use: "source filedata-compil-env.bash" before compilation step
##############################################################################

echo " "
echo "---------------------------------------"
echo "-- filedata compilation environment  --"
echo "---------------------------------------"

#-------------------------#
# HDF library information #
#-------------------------#

export HDFINC="-I$EXPLOIT_HOME/include"
export HDFLIBDIR="-L$EXPLOIT_HOME/lib"
export HDFLIBS="-lmfhdf -ldf -ljpeg -lz -lm"

#--- Developping purpose ---
MACHINE_NAME=`uname -n`
case $MACHINE_NAME in
    "pascal-workstation" )
        export HDFINC="-I/usr/local/hdf4/include"
        export HDFLIBDIR="-L/usr/local/hdf4/lib"
        ;;
    "icarnassier.icare.univ-lille1.fr" )
        export HDFINC="-I/usr/ops/env64/opt/hdf4/include"
        export HDFLIBDIR="-L/usr/ops/env64/opt/hdf4/lib"
        ;;
    "icare63.icare.univ-lille1.fr" )
        export HDFINC="-I/home/pascal/ops/env32-rhel6/opt/hdf4/include"
        export HDFLIBDIR="-L/home/pascal/ops/env32-rhel6/opt/hdf4/lib"
        ;;
    "icare54.icare.univ-lille1.fr" )
        export HDFINC="-I/usr/ops/env64/opt/hdf4/include"
        export HDFLIBDIR="-L/usr/ops/env64/opt/hdf4/lib"
        ;;
    "camelot.ipsl.polytechnique.fr" )
        export HDFINC="-I/usr/include/hdf"
        export HDFLIBDIR="-L/usr/lib64/hdf"
        ;;
esac

echo " HDF includes      :" $HDFINC
echo " HDF libs dir      :" $HDFLIBDIR
echo " HDF libs          :" $HDFLIBS

#------------------------------#
#  hdfcpp library information  #
#------------------------------#

export HDFCPPINC="-I$EXPLOIT_HOME/include"
export HDFCPPLIBDIR="-L$EXPLOIT_HOME/lib"
export HDFCPPLIBS="-lhdfcpp"

#--- Developping purpose ---
case $MACHINE_NAME in
    "pascal-workstation" )
        export HDFCPPINC="-I$HOME/depot/hdfcpp/include"
        export HDFCPPLIBDIR="-L$HOME/depot/hdfcpp/src"
        ;;
    "icare63.icare.univ-lille1.fr" )
        if [[ $USER == "pascal" ]] ; then
            export HDFCPPINC="-I$HOME/depot/hdfcpp/include"
            export HDFCPPLIBDIR="-L$HOME/depot/hdfcpp/src"
        fi
        ;;
    "icarnassier.icare.univ-lille1.fr" )
        export HDFCPPINC="-I$HOME/depot/hdfcpp/include"
        export HDFCPPLIBDIR="-L$HOME/depot/hdfcpp/src"
        ;;
    "icare54.icare.univ-lille1.fr" )
        export HDFCPPINC="-I$HOME/depot/hdfcpp/include"
        export HDFCPPLIBDIR="-L$HOME/depot/hdfcpp/src"
        ;;
esac

echo " hdfcpp includes      :" $HDFCPPINC
echo " hdfcpp libs dir      :" $HDFCPPLIBDIR
echo " hdfcpp libs          :" $HDFCPPLIBS

#----------------------------#
#  HDF5 library information  #
#----------------------------#

export HDF5LIBS="-lhdf5"
export HDF5INC="-I$EXPLOIT_HOME/include"
export HDF5LIBDIR="-L$EXPLOIT_HOME/lib"

case $MACHINE_NAME in
    "pascal-workstation" )
        export HDF5INC="-I/usr/local/hdf5/include"
        export HDF5LIBDIR="-L/usr/local/hdf5/lib"
        ;;
    "icarnassier.icare.univ-lille1.fr" )
        export HDF5INC="-I/usr/ops/env64/include"
        export HDF5LIBDIR="-L/usr/ops/env64/lib"
        ;;
    "icare54.icare.univ-lille1.fr" )
        export HDF5INC="-I/usr/ops/env64/include"
        export HDF5LIBDIR="-L/usr/ops/env64/lib"
        ;;
    "icare63.icare.univ-lille1.fr" )
        export HDF5INC="-I/home/pascal/ops/env32-rhel6/include"
        export HDF5LIBDIR="-L/home/pascal/ops/env32-rhel6/lib"
        ;;
esac

echo " HDF5 includes      :" $HDF5INC
echo " HDF5 libs dir      :" $HDF5LIBDIR
echo " HDF5 libs          :" $HDF5LIBS

#--------------------------------#
#  GRIB API library information  #
#--------------------------------#

export GRIBINC="-I$EXPLOIT_HOME/include"
export GRIBLIBDIR="-L$EXPLOIT_HOME/lib"
export GRIBLIBS="-lgrib_api -ljasper"

case $MACHINE_NAME in
    "pascal-workstation" )
        export GRIBINC="-I/usr/local/include"
        export GRIBLIBDIR="-L/usr/local/lib"
        ;;
    "icarnassier.icare.univ-lille1.fr" )
        export GRIBINC="-I/usr/ops/env64/include"
        export GRIBLIBDIR="-L/usr/ops/env64/lib"
        ;;
    "icare54.icare.univ-lille1.fr" )
        export GRIBINC="-I/usr/ops/env64/include"
        export GRIBLIBDIR="-L/usr/ops/env64/lib"
        ;;
    "icare63.icare.univ-lille1.fr" )
        export GRIBINC="-I/home/pascal/ops/env32-rhel6/include"
        export GRIBLIBDIR="-L/home/pascal/ops/env32-rhel6/lib"
        ;;
esac

echo " GRIB includes      :" $GRIBINC
echo " GRIB libs dir      :" $GRIBLIBDIR
echo " GRIB libs          :" $GRIBLIBS

#-------------------------------#
#    GSL library information    #
#-------------------------------#

export GSLINC="-I$EXPLOIT_HOME/include"
export GSLLIBDIR="-L$EXPLOIT_HOME/lib"
export GSLLIBS="-lgsl -lgslcblas"

case $MACHINE_NAME in
    "pascal-workstation" )
        export GSLINC="-I/usr/local/include"
        export GSLLIBDIR="-L/usr/local/lib"
        ;;
    "icarnassier.icare.univ-lille1.fr" )
        export GSLINC="-I/usr/ops/env64/include"
        export GSLLIBDIR="-L/usr/ops/env64/lib"
        ;;
    "icare54.icare.univ-lille1.fr" )
        export GSLINC="-I/usr/ops/env64/include"
        export GSLLIBDIR="-L/usr/ops/env64/lib"
        ;;
    "icare63.icare.univ-lille1.fr" )
        export GSLINC="-I/home/pascal/ops/env32-rhel6/include"
        export GSLLIBDIR="-L/home/pascal/ops/env32-rhel6/lib"
        ;;
esac

echo " GSL includes      :" $GSLINC
echo " GSL libs dir      :" $GSLLIBDIR
echo " GSL libs          :" $GSLLIBS

echo " "
